@@ -69,6 +69,7 @@ class VMUtilsTestCase(base.BaseTestCase):
         self._vmutils = vmutils.VMUtils()
         self._vmutils._conn = mock.MagicMock()
         self._vmutils._jobutils = mock.MagicMock()
+        self._vmutils._pathutils = mock.MagicMock()
 
         super(VMUtilsTestCase, self).setUp()
 
@@ -132,20 +133,18 @@ class VMUtilsTestCase(base.BaseTestCase):
         self._test_set_vm_memory_dynamic(2.0)
 
     def _test_set_vm_memory_dynamic(self, dynamic_memory_ratio):
-        mock_vm = self._lookup_vm()
-
         mock_s = self._vmutils._conn.Msvm_VirtualSystemSettingData()[0]
         mock_s.SystemType = 3
 
         mock_vmsetting = mock.MagicMock()
         mock_vmsetting.associators.return_value = [mock_s]
 
-        self._vmutils._set_vm_memory(mock_vm, mock_vmsetting,
+        self._vmutils._set_vm_memory(mock_vmsetting,
                                      self._FAKE_MEMORY_MB,
                                      dynamic_memory_ratio)
 
         self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
-            mock_s, mock_vm)
+            mock_s)
 
         if dynamic_memory_ratio > 1:
             self.assertTrue(mock_s.DynamicMemoryEnabled)
@@ -247,12 +246,10 @@ class VMUtilsTestCase(base.BaseTestCase):
                                 mock.sentinel.instance_path)
 
         self.assertTrue(getattr(mock_svc, self._DEFINE_SYSTEM).called)
-        mock_set_mem.assert_called_with(mock_vm, mock_s, self._FAKE_MEMORY_MB,
+        mock_set_mem.assert_called_with(mock_s, self._FAKE_MEMORY_MB,
                                         self._FAKE_DYNAMIC_MEMORY_RATIO)
 
-        mock_set_vcpus.assert_called_with(mock_vm, mock_s,
-                                          self._FAKE_VCPUS_NUM,
-                                          False)
+        mock_set_vcpus.assert_called_with(mock_s, self._FAKE_VCPUS_NUM, False)
 
     def test_get_vm_scsi_controller(self):
         self._prepare_get_vm_controller(self._vmutils._SCSI_CTRL_RES_SUB_TYPE)
@@ -276,7 +273,7 @@ class VMUtilsTestCase(base.BaseTestCase):
     def test_get_free_controller_slot_exception(self):
         fake_drive = mock.MagicMock()
         type(fake_drive).AddressOnParent = mock.PropertyMock(
-            side_effect=range(constants.SCSI_CONTROLLER_SLOTS_NUMBER))
+            side_effect=list(range(constants.SCSI_CONTROLLER_SLOTS_NUMBER)))
 
         with mock.patch.object(self._vmutils,
                 'get_attached_disks') as fake_get_attached_disks:
@@ -384,14 +381,13 @@ class VMUtilsTestCase(base.BaseTestCase):
 
     @mock.patch.object(vmutils.VMUtils, '_get_nic_data_by_name')
     def test_destroy_nic(self, mock_get_nic_data_by_name):
-        mock_vm = self._lookup_vm()
         mock_nic_data = mock_get_nic_data_by_name.return_value
 
         self._vmutils.destroy_nic(self._FAKE_VM_NAME,
                                   mock.sentinel.FAKE_NIC_NAME)
 
         self._vmutils._jobutils.remove_virt_resource.assert_called_once_with(
-            mock_nic_data, mock_vm)
+            mock_nic_data)
 
     def test_set_vm_state(self):
         mock_vm = self._lookup_vm()
@@ -416,7 +412,7 @@ class VMUtilsTestCase(base.BaseTestCase):
             self._FAKE_VM_PATH)
 
     def test_set_disk_host_resource(self):
-        mock_vm = self._lookup_vm()
+        self._lookup_vm()
         mock_rasds = self._create_mock_disks()
 
         self._vmutils._get_vm_disks = mock.MagicMock(
@@ -432,7 +428,7 @@ class VMUtilsTestCase(base.BaseTestCase):
         self._vmutils._get_disk_resource_address.assert_called_with(
             mock_rasds[0])
         self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
-            mock_rasds[0], mock_vm)
+            mock_rasds[0])
         self.assertEqual(
             mock.sentinel.fake_new_mounted_disk_path,
             mock_rasds[0].HostResource[0])
@@ -511,14 +507,25 @@ class VMUtilsTestCase(base.BaseTestCase):
         ret_val = self._vmutils.get_vm_dvd_disk_paths(self._FAKE_VM_NAME)
         self.assertEqual(mock.sentinel.FAKE_DVD_PATH1, ret_val[0])
 
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_is_disk_attached(self, mock_get_mounted_disk_from_path):
+        is_physical = True
+
+        is_attached = self._vmutils.is_disk_attached(mock.sentinel.disk_path,
+                                                     is_physical=is_physical)
+
+        self.assertTrue(is_attached)
+        mock_get_mounted_disk_from_path.assert_called_once_with(
+            mock.sentinel.disk_path, is_physical)
+
     def test_detach_vm_disk(self):
-        mock_vm = self._lookup_vm()
         mock_disk = self._prepare_mock_disk()
 
         self._vmutils.detach_vm_disk(self._FAKE_VM_NAME,
                                      self._FAKE_HOST_RESOURCE)
         self._vmutils._jobutils.remove_virt_resource.assert_called_once_with(
-            mock_disk, mock_vm)
+            mock_disk)
 
     def test_get_mounted_disk_resource_from_path(self):
         mock_disk_1 = mock.MagicMock()
@@ -588,8 +595,7 @@ class VMUtilsTestCase(base.BaseTestCase):
 
         if new_connection:
             self.assertEqual(new_connection, ret_val)
-            fake_modify.assert_called_once_with(fake_serial_port,
-                                                mock_vm)
+            fake_modify.assert_called_once_with(fake_serial_port)
         else:
             self.assertEqual(old_serial_connection, ret_val)
 
@@ -617,20 +623,17 @@ class VMUtilsTestCase(base.BaseTestCase):
     def test_modify_virtual_system(self):
         mock_vs_man_svc = mock.MagicMock()
         mock_vmsetting = mock.MagicMock()
-        fake_path = 'fake path'
         fake_job_path = 'fake job path'
         fake_ret_val = 'fake return value'
 
-        mock_vs_man_svc.ModifyVirtualSystem.return_value = (0, fake_job_path,
+        mock_vs_man_svc.ModifySystemSettings.return_value = (fake_job_path,
                                                             fake_ret_val)
 
         self._vmutils._modify_virtual_system(vs_man_svc=mock_vs_man_svc,
-                                             vm_path=fake_path,
                                              vmsetting=mock_vmsetting)
 
-        mock_vs_man_svc.ModifyVirtualSystem.assert_called_once_with(
-            ComputerSystem=fake_path,
-            SystemSettingData=mock_vmsetting.GetText_(1))
+        mock_vs_man_svc.ModifySystemSettings.assert_called_once_with(
+            SystemSettings=mock_vmsetting.GetText_(1))
         self._vmutils._jobutils.check_ret_val.assert_called_once_with(
             fake_ret_val, fake_job_path)
 
@@ -852,4 +855,138 @@ class VMUtilsTestCase(base.BaseTestCase):
             mock_set_secure_boot.assert_called_once_with(
                 vs_data, mock.sentinel.msft_ca_required)
             mock_modify_virtual_system.assert_called_once_with(
-                vs_svc, vm.path_(), vs_data)
+                vs_svc, vs_data)
+
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_set_disk_qos_specs(self, mock_get_disk_resource):
+        mock_disk = mock_get_disk_resource.return_value
+
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path,
+                                         max_iops=mock.sentinel.max_iops,
+                                         min_iops=mock.sentinel.min_iops)
+
+        mock_get_disk_resource.assert_called_once_with(
+            mock.sentinel.disk_path, is_physical=False)
+        self.assertEqual(mock.sentinel.max_iops, mock_disk.IOPSLimit)
+        self.assertEqual(mock.sentinel.min_iops, mock_disk.IOPSReservation)
+        self._vmutils._jobutils.modify_virt_resource.assert_called_once_with(
+            mock_disk)
+
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_set_disk_qos_specs_missing_values(self, mock_get_disk_resource):
+        self._vmutils.set_disk_qos_specs(mock.sentinel.disk_path)
+
+        self.assertFalse(mock_get_disk_resource.called)
+
+    def _test_is_drive_physical(self, is_physical):
+        self._vmutils._pathutils.exists.return_value = not is_physical
+        ret = self._vmutils._is_drive_physical(mock.sentinel.fake_drive_path)
+
+        self.assertEqual(is_physical, ret)
+
+    def test_is_drive_phyisical_true(self):
+        self._test_is_drive_physical(is_physical=True)
+
+    def test_is_drive_physical_false(self):
+        self._test_is_drive_physical(is_physical=False)
+
+    @mock.patch.object(vmutils.VMUtils, '_is_drive_physical')
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    @mock.patch.object(vmutils, 'wmi', create=True)
+    def _test_drive_to_boot_source(self, mock_wmi, mock_get_disk_res_from_path,
+                                  mock_is_drive_physical, is_physical):
+        mock_is_drive_physical.return_value = is_physical
+        mock_drive = mock.MagicMock(Parent=mock.sentinel.fake_drive_parent)
+        mock_drive.associators.return_value = [mock.sentinel.physical_bssd]
+        mock_get_disk_res_from_path.return_value = mock_drive
+        mock_rads = mock.MagicMock()
+        mock_rads.associators.return_value = [mock.sentinel.bssd]
+        mock_wmi.WMI.return_value = mock_rads
+
+        ret = self._vmutils._drive_to_boot_source(mock.sentinel.drive_path)
+
+        mock_is_drive_physical.assert_called_once_with(
+            mock.sentinel.drive_path)
+        mock_get_disk_res_from_path.assert_called_once_with(
+            mock.sentinel.drive_path, is_physical=is_physical)
+        if is_physical:
+            self.assertEqual(mock.sentinel.physical_bssd, ret)
+        else:
+            self.assertEqual(mock.sentinel.bssd, ret)
+
+    def test_physical_drive_to_boot_source(self):
+        self._test_drive_to_boot_source(is_physical=True)
+
+    def test_drive_to_boot_source(self):
+        self._test_drive_to_boot_source(is_physical=False)
+
+    @mock.patch.object(vmutils.VMUtils, '_set_boot_order_gen1')
+    @mock.patch.object(vmutils.VMUtils, '_set_boot_order_gen2')
+    @mock.patch.object(vmutils.VMUtils, 'get_vm_generation')
+    def _test_set_boot_order(self, mock_get_vm_gen, mock_set_boot_order_gen2,
+                             mock_set_boot_order_gen1, vm_gen):
+        mock_get_vm_gen.return_value = vm_gen
+        self._vmutils.set_boot_order(mock.sentinel.fake_vm_name,
+                                     mock.sentinel.boot_order)
+        if vm_gen == constants.VM_GEN_1:
+            mock_set_boot_order_gen1.assert_called_once_with(
+                mock.sentinel.fake_vm_name, mock.sentinel.boot_order)
+        else:
+            mock_set_boot_order_gen2.assert_called_once_with(
+                mock.sentinel.fake_vm_name, mock.sentinel.boot_order)
+
+    def test_set_boot_order_gen1_vm(self):
+        self._test_set_boot_order(vm_gen=constants.VM_GEN_1)
+
+    def test_set_boot_order_gen2_vm(self):
+        self._test_set_boot_order(vm_gen=constants.VM_GEN_2)
+
+    @mock.patch.object(vmutils.VMUtils, '_get_vm_setting_data')
+    @mock.patch.object(vmutils.VMUtils, '_modify_virtual_system')
+    def test_set_boot_order_gen1(self, mock_modify_virt_syst,
+                            mock_get_vm_setting_data):
+        mock_vm = self._lookup_vm()
+
+        mock_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
+        mock_vssd = mock_get_vm_setting_data.return_value
+        fake_dev_boot_order = [mock.sentinel.BOOT_DEV1,
+                               mock.sentinel.BOOT_DEV2]
+        self._vmutils._set_boot_order_gen1(
+            mock_vm.name, fake_dev_boot_order)
+
+        mock_modify_virt_syst.assert_called_once_with(
+            mock_svc, mock_vssd)
+        self.assertEqual(mock_vssd.BootOrder, tuple(fake_dev_boot_order))
+
+    @mock.patch.object(vmutils.VMUtils, '_get_vm_setting_data')
+    @mock.patch.object(vmutils.VMUtils, '_drive_to_boot_source')
+    @mock.patch.object(vmutils.VMUtils, '_modify_virtual_system')
+    def test_set_boot_order_gen2(self, mock_modify_virtual_system,
+                                 mock_drive_to_boot_source,
+                                 mock_get_vm_setting_data):
+        fake_boot_dev1 = mock.MagicMock()
+        fake_boot_dev2 = mock.MagicMock()
+        fake_boot_dev1.path_.return_value = mock.sentinel.BOOT_SOURCE1
+        fake_boot_dev2.path_.return_value = mock.sentinel.BOOT_SOURCE2
+
+        fake_dev_order = [fake_boot_dev1, fake_boot_dev2]
+        mock_drive_to_boot_source.side_effect = fake_dev_order
+        mock_vm = self._lookup_vm()
+        mock_svc = self._vmutils._conn.Msvm_VirtualSystemManagementService()[0]
+        mock_vssd = mock_get_vm_setting_data.return_value
+        old_boot_order = tuple([mock.sentinel.BOOT_SOURCE2,
+                                mock.sentinel.BOOT_SOURCE1,
+                                mock.sentinel.BOOT_SOURCE_NET])
+        expected_boot_order = tuple([mock.sentinel.BOOT_SOURCE1,
+                                     mock.sentinel.BOOT_SOURCE2,
+                                     mock.sentinel.BOOT_SOURCE_NET])
+        mock_vssd.BootSourceOrder = old_boot_order
+
+        self._vmutils._set_boot_order_gen2(mock_vm.name, fake_dev_order)
+
+        mock_modify_virtual_system.assert_called_once_with(
+            mock_svc, mock_vssd)
+        self.assertEqual(expected_boot_order, mock_vssd.BootSourceOrder)
