@@ -37,6 +37,7 @@ from os_win import exceptions
 from os_win.utils import constants
 from os_win.utils import hostutils
 from os_win.utils import jobutils
+from os_win.utils import pathutils
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
@@ -65,6 +66,7 @@ class VMUtils(object):
         'Msvm_SyntheticEthernetPortSettingData')
     _AFFECTED_JOB_ELEMENT_CLASS = "Msvm_AffectedJobElement"
     _COMPUTER_SYSTEM_CLASS = "Msvm_ComputerSystem"
+    _LOGICAL_IDENTITY_CLASS = 'Msvm_LogicalIdentity'
 
     _VIRTUAL_SYSTEM_SUBTYPE = 'VirtualSystemSubType'
     _VIRTUAL_SYSTEM_TYPE_REALIZED = 'Microsoft:Hyper-V:System:Realized'
@@ -90,6 +92,7 @@ class VMUtils(object):
 
     def __init__(self, host='.'):
         self._jobutils = jobutils.JobUtils()
+        self._pathutils = pathutils.PathUtils()
         self._enabled_states_map = {v: k for k, v in
                                     six.iteritems(self._vm_power_states_map)}
         if sys.platform == 'win32':
@@ -144,10 +147,10 @@ class VMUtils(object):
         si = summary_info[0]
         memory_usage = None
         if si.MemoryUsage is not None:
-            memory_usage = long(si.MemoryUsage)
+            memory_usage = int(si.MemoryUsage)
         up_time = None
         if si.UpTime is not None:
-            up_time = long(si.UpTime)
+            up_time = int(si.UpTime)
 
         # Nova requires a valid state to be returned. Hyper-V has more
         # states than Nova, typically intermediate ones and since there is
@@ -199,18 +202,18 @@ class VMUtils(object):
         return [s for s in vmsettings if
                 s.VirtualSystemType == self._VIRTUAL_SYSTEM_TYPE_REALIZED][0]
 
-    def _set_vm_memory(self, vm, vmsetting, memory_mb, dynamic_memory_ratio):
+    def _set_vm_memory(self, vmsetting, memory_mb, dynamic_memory_ratio):
         mem_settings = vmsetting.associators(
             wmi_result_class=self._MEMORY_SETTING_DATA_CLASS)[0]
 
-        max_mem = long(memory_mb)
+        max_mem = int(memory_mb)
         mem_settings.Limit = max_mem
 
         if dynamic_memory_ratio > 1:
             mem_settings.DynamicMemoryEnabled = True
             # Must be a multiple of 2
             reserved_mem = min(
-                long(max_mem / dynamic_memory_ratio) >> 1 << 1,
+                int(max_mem / dynamic_memory_ratio) >> 1 << 1,
                 max_mem)
         else:
             mem_settings.DynamicMemoryEnabled = False
@@ -220,25 +223,25 @@ class VMUtils(object):
         # Start with the minimum memory
         mem_settings.VirtualQuantity = reserved_mem
 
-        self._jobutils.modify_virt_resource(mem_settings, vm)
+        self._jobutils.modify_virt_resource(mem_settings)
 
-    def _set_vm_vcpus(self, vm, vmsetting, vcpus_num, limit_cpu_features):
+    def _set_vm_vcpus(self, vmsetting, vcpus_num, limit_cpu_features):
         procsetting = vmsetting.associators(
             wmi_result_class=self._PROCESSOR_SETTING_DATA_CLASS)[0]
-        vcpus = long(vcpus_num)
+        vcpus = int(vcpus_num)
         procsetting.VirtualQuantity = vcpus
         procsetting.Reservation = vcpus
         procsetting.Limit = 100000  # static assignment to 100%
         procsetting.LimitProcessorFeatures = limit_cpu_features
 
-        self._jobutils.modify_virt_resource(procsetting, vm)
+        self._jobutils.modify_virt_resource(procsetting)
 
     def update_vm(self, vm_name, memory_mb, vcpus_num, limit_cpu_features,
                   dynamic_memory_ratio):
         vm = self._lookup_vm_check(vm_name)
         vmsetting = self._get_vm_setting_data(vm)
-        self._set_vm_memory(vm, vmsetting, memory_mb, dynamic_memory_ratio)
-        self._set_vm_vcpus(vm, vmsetting, vcpus_num, limit_cpu_features)
+        self._set_vm_memory(vmsetting, memory_mb, dynamic_memory_ratio)
+        self._set_vm_vcpus(vmsetting, vcpus_num, limit_cpu_features)
 
     def check_admin_permissions(self):
         if not self._conn.Msvm_VirtualSystemManagementService():
@@ -256,10 +259,10 @@ class VMUtils(object):
         vmsetting = self._get_vm_setting_data(vm)
 
         LOG.debug('Setting memory for vm %s', vm_name)
-        self._set_vm_memory(vm, vmsetting, memory_mb, dynamic_memory_ratio)
+        self._set_vm_memory(vmsetting, memory_mb, dynamic_memory_ratio)
 
         LOG.debug('Set vCPUs for vm %s', vm_name)
-        self._set_vm_vcpus(vm, vmsetting, vcpus_num, limit_cpu_features)
+        self._set_vm_vcpus(vmsetting, vcpus_num, limit_cpu_features)
 
     def _create_vm_obj(self, vs_man_svc, vm_name, vm_gen, notes,
                        dynamic_memory_ratio, instance_path):
@@ -295,10 +298,9 @@ class VMUtils(object):
             vm_path = job.associators(self._AFFECTED_JOB_ELEMENT_CLASS)[0]
         return self._get_wmi_obj(vm_path)
 
-    def _modify_virtual_system(self, vs_man_svc, vm_path, vmsetting):
-        (job_path, ret_val) = vs_man_svc.ModifyVirtualSystem(
-            ComputerSystem=vm_path,
-            SystemSettingData=vmsetting.GetText_(1))[1:]
+    def _modify_virtual_system(self, vs_man_svc, vmsetting):
+        (job_path, ret_val) = vs_man_svc.ModifySystemSettings(
+            SystemSettings=vmsetting.GetText_(1))
         self._jobutils.check_ret_val(ret_val, job_path)
 
     def get_vm_scsi_controller(self, vm_name):
@@ -454,7 +456,7 @@ class VMUtils(object):
                               {'old': disk_resource.HostResource[0],
                                'new': mounted_disk_path})
                     disk_resource.HostResource = [mounted_disk_path]
-                    self._jobutils.modify_virt_resource(disk_resource, vm)
+                    self._jobutils.modify_virt_resource(disk_resource)
                 disk_found = True
                 break
         if not disk_found:
@@ -491,10 +493,9 @@ class VMUtils(object):
         :param vm_name: The name of the VM which has the NIC to be destroyed.
         :param nic_name: The NIC's ElementName.
         """
+        # TODO(claudiub): remove vm_name argument, no longer used.
         nic_data = self._get_nic_data_by_name(nic_name)
-
-        vm = self._lookup_vm_check(vm_name)
-        self._jobutils.remove_virt_resource(nic_data, vm)
+        self._jobutils.remove_virt_resource(nic_data)
 
     def soft_shutdown_vm(self, vm_name):
         vm = self._lookup_vm_check(vm_name)
@@ -633,8 +634,13 @@ class VMUtils(object):
 
         return dvd_paths
 
+    def is_disk_attached(self, disk_path, is_physical=True):
+        disk_resource = self._get_mounted_disk_resource_from_path(disk_path,
+                                                                  is_physical)
+        return disk_resource is not None
+
     def detach_vm_disk(self, vm_name, disk_path, is_physical=True):
-        vm = self._lookup_vm_check(vm_name)
+        # TODO(claudiub): remove vm_name argument, no longer used.
         disk_resource = self._get_mounted_disk_resource_from_path(disk_path,
                                                                   is_physical)
 
@@ -644,9 +650,9 @@ class VMUtils(object):
                                       "WHERE __PATH = '%s'" %
                                       disk_resource.Parent)[0]
 
-            self._jobutils.remove_virt_resource(disk_resource, vm)
+            self._jobutils.remove_virt_resource(disk_resource)
             if not is_physical:
-                self._jobutils.remove_virt_resource(parent, vm)
+                self._jobutils.remove_virt_resource(parent)
 
     def _get_mounted_disk_resource_from_path(self, disk_path, is_physical):
         if is_physical:
@@ -716,7 +722,7 @@ class VMUtils(object):
 
         if update_connection:
             serial_port.Connection = [update_connection]
-            self._jobutils.modify_virt_resource(serial_port, vm)
+            self._jobutils.modify_virt_resource(serial_port)
 
         if len(serial_port.Connection) > 0:
             return serial_port.Connection[0]
@@ -806,7 +812,7 @@ class VMUtils(object):
         vs_data = self._get_vm_setting_data(vm)
         self._set_secure_boot(vs_data, msft_ca_required)
         vs_man_svc = self._conn.Msvm_VirtualSystemManagementService()[0]
-        self._modify_virtual_system(vs_man_svc, vm.path_(), vs_data)
+        self._modify_virtual_system(vs_man_svc, vs_data)
 
     def _set_secure_boot(self, vs_data, msft_ca_required):
         vs_data.SecureBootEnabled = True
@@ -814,3 +820,66 @@ class VMUtils(object):
             raise exceptions.HyperVException(
                 _('UEFI SecureBoot is supported only on Windows instances for '
                   'this Hyper-V version.'))
+
+    def set_disk_qos_specs(self, disk_path, max_iops=None, min_iops=None):
+        if min_iops is None and max_iops is None:
+            LOG.debug("Skipping setting disk QoS specs as no "
+                      "value was provided.")
+            return
+
+        disk_resource = self._get_mounted_disk_resource_from_path(
+            disk_path, is_physical=False)
+
+        if max_iops is not None:
+            disk_resource.IOPSLimit = max_iops
+        if min_iops is not None:
+            disk_resource.IOPSReservation = min_iops
+
+        self._jobutils.modify_virt_resource(disk_resource)
+
+    def _is_drive_physical(self, drive_path):
+        # TODO(atuvenie): Find better way to check if path represents
+        # physical or virtual drive.
+        return not self._pathutils.exists(drive_path)
+
+    def _drive_to_boot_source(self, drive_path):
+        is_physical = self._is_drive_physical(drive_path)
+        drive = self._get_mounted_disk_resource_from_path(
+            drive_path, is_physical=is_physical)
+
+        if is_physical:
+            bssd = drive.associators(
+                wmi_association_class=self._LOGICAL_IDENTITY_CLASS)[0]
+        else:
+            rasd = wmi.WMI(moniker=drive.Parent)
+            bssd = rasd.associators(
+                wmi_association_class=self._LOGICAL_IDENTITY_CLASS)[0]
+        return bssd
+
+    def set_boot_order(self, vm_name, device_boot_order):
+        if self.get_vm_generation(vm_name) == constants.VM_GEN_1:
+            self._set_boot_order_gen1(vm_name, device_boot_order)
+        else:
+            self._set_boot_order_gen2(vm_name, device_boot_order)
+
+    def _set_boot_order_gen1(self, vm_name, device_boot_order):
+        vm = self._lookup_vm_check(vm_name)
+
+        vs_man_svc = self._conn.Msvm_VirtualSystemManagementService()[0]
+        vssd = self._get_vm_setting_data(vm)
+        vssd.BootOrder = tuple(device_boot_order)
+
+        self._modify_virtual_system(vs_man_svc, vssd)
+
+    def _set_boot_order_gen2(self, vm_name, device_boot_order):
+        new_boot_order = [(self._drive_to_boot_source(device)).path_()
+                           for device in device_boot_order if device]
+
+        vm = self._lookup_vm_check(vm_name)
+
+        vs_man_svc = self._conn.Msvm_VirtualSystemManagementService()[0]
+        vssd = self._get_vm_setting_data(vm)
+        old_boot_order = vssd.BootSourceOrder
+        network_boot_devs = set(old_boot_order) ^ set(new_boot_order)
+        vssd.BootSourceOrder = tuple(new_boot_order) + tuple(network_boot_devs)
+        self._modify_virtual_system(vs_man_svc, vssd)
