@@ -69,7 +69,7 @@ class ISCSITargetUtils(object):
         wt_portals = self._conn_wmi.WT_Portal()
 
         if available_only:
-            wt_portals = filter(lambda portal: portal.Listen, wt_portals)
+            wt_portals = list(filter(lambda portal: portal.Listen, wt_portals))
 
         if not wt_portals and fail_if_none_found:
             err_msg = _("No valid iSCSI portal was found.")
@@ -100,7 +100,7 @@ class ISCSITargetUtils(object):
             err_msg = _('Could not find WT Disk: %s')
             raise exceptions.ISCSITargetException(err_msg % description)
 
-    def _get_wt_snapshot(self, description, fail_if_not_found):
+    def _get_wt_snapshot(self, description, fail_if_not_found=True):
         wt_snapshots = self._conn_wmi.WT_Snapshot(Description=description)
         if wt_snapshots:
             return wt_snapshots[0]
