@@ -104,11 +104,11 @@ utils_map = {
 
 def _get_class(class_type):
     if class_type not in utils_map:
-        raise exceptions.HyperVException(_('Class type %(class)s does '
+        raise exceptions.HyperVException(_('Class type %s does '
                                            'not exist') % class_type)
 
     windows_version = utils.get_windows_version()
-    build = map(int, windows_version.split('.'))
+    build = list(map(int, windows_version.split('.')))
     windows_version = float("%i.%i" % (build[0], build[1]))
 
     existing_classes = utils_map.get(class_type)
